// -------------------------------------------------------------------------- //
// Built-In Component Templates
//
// Template            Family              Properties
// --------            ------              ---------
// Animation           Animation           frames, delay, duration, looped, global
// Collidable          Collidable
// GravityClipped      GravityClipped
// Location            Location            x, y, width, height
// Physical            Physical            frictionX, frictionY, gravity
// MessageScript       MessageScript       handlers, * (for namespace)
// TimerScript         TimerScript         OnExpired, delay, * (for namespace)
// TileSensor          TileSensor
// Velocity            Velocity            vx, vy, vxMax, vyMax
// Visible             Visible             lib, frame
// ZeroGClipped        ZeroGClipped
//
// ArachnutAI          ArachnutAI
// BounderAI           BounderAI
// CloudAI             CloudAI
// InchwormAI          InchwormAI
// KeenBehavior        KeenBehavior
// LickAI              LickAI
// LittleKeenBehavior  LittleKeenBehavior
// MimrockAI           MimrockAI
// MushroomAI          MushroomAI
// SlugAI              SlugAI
// SnakeAI             SnakeAI
// -------------------------------------------------------------------------- //
[

// A generic component that calls an OnCollided function when its entity
// collides with another entity. It provides a simpler interface for
// specialization to components that need to react to contact with an entity.
//
// Property     Type                    Description
// --------     ----                    -----------
// OnCollided   function(other : id)    The function to be called
//
{"Component": "OnCollided", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "Collided":
                function(args : association)
                    OnCollided(args["eid"]);
                end function
        },
        "OnCollided":
            function(other : id)
                debug("Entity " & image(this) & " does not implement OnCollided");
            end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A generic component that calls an OnDestruct function just before its
// entity is destroyed, notified by the "Destruct" message. It provides a
// simpler interface for specialization of components that need to react to
// entity destruction.
//
// Property     Type          Description
// --------     ----          -----------
// OnDestruct   function()    The function to be called
//
{"Component": "OnDestruct", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "Destruct":
                function()
                    OnDestruct();
                end function
        },
        "OnDestruct":
            function()
                debug("Entity " & image(this) & " does not implement OnDestruct");
            end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A generic component that calls an OnSpawn function just after its entity
// is spawned into the world, notified by the "Spawned" message. It provides a
// simpler interface for specialization of components that need to do
// something when an entity is created.
//
// Property     Type          Description
// --------     ----          -----------
// OnSpawned    function()    The function to be called
//
{"Component": "OnSpawned", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "Spawned":
                function()
                    OnSpawned();
                end function
        },
        "OnSpawned":
            function()
                debug("Entity " & image(this) & " does not implement OnSpawned");
            end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A generic component that calls an OnWorldLoaded function just after the
// entity's world is loaded for gameplay, as notified by the "WorldLoaded"
// message. This can occur multiple times over the lifespan of the component:
// when the world is loaded in the editor, when the player enters the world
// during a game session, when a saved game is loaded with the world in play,
// etc. This component provides a simpler interface for specialization of
// components that need to do something when the world is loaded (ex: react to
// a level being beaten.)
//
// Property        Type         Description
// --------        ----         -----------
// OnWorldLoaded   function()   The function to be called
//
{"Component": "OnWorldLoaded", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "WorldLoaded":
                function()
                    OnWorldLoaded();
                end function
        },
        "OnWorldLoaded":
            function()
                debug("Entity " & image(this) & " does not implement OnWorldLoaded");
            end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that indicates its entity can be Activated, by sending an
// "ActivateOnTouch" message to entities on contact.
//
{"Component": "Activator", "Specializes": "OnCollided",
    "Properties": {
        "OnCollided": 
            function(other : id)
                engine::Send(other, "ActivateOnTouch", {"eid": this});
            end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that indicates its entity is climbable, by sending a "Climb"
// message in response to a "TryToClimb" message.
//
{"Component": "Climbable", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "TryToClimb":
                function(args : association)
                    engine::Send(args["eid"], "Climb", {"eid": this});
                end function
        }
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that gives a bonus when its entity makes contact with an
// ItemCollector's entity. After the item's bonus is collected, it self
// destructs.
//
// Property     Type        Description
// --------     ----        -----------
// name         string      The name of the game value to receive a bonus.
// amount       number      Amount to add to the game value.
//
{"Component": "Item", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "Collided":
                function(args : association)
                    engine::Send(args["eid"], "CollectItem", {"eid": this, "name": name, "amount": amount});
                end function,
            "ItemCollected":
                function()
                    engine::Destroy(this);
                end function
        }
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that reacts to Item components on contact, by collecting the
// bonus and notifying the Item that it has been collected.
//
{"Component": "ItemCollector", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "CollectItem":
                function(args : association)
                    game::[args["name"]] += args["amount"];
                    engine::Send(args["eid"], "ItemCollected");
                end function
        }
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that is lethal on contact, to a certain type of entity. It
// sends a Kill message on contact.
//
// Property    Type       Description
// --------    ----       -----------
// victim      string     The type of entity to kill
//
{"Component": "Lethal", "Specializes": "OnCollided",
    "Properties": {
        "OnCollided":
            function(other : id)
                engine::Send(other, "Kill", {"victim": victim});
            end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that is lethal on contact to Player entities.
//
{"Component": "LethalToPlayer", "Specializes": "Lethal",
    "Properties": {
        "victim": "Player"
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A Physical component parameterized for Keen's platform levels.
//
{"Component": "PhysicalPlatform", "Specializes": "Physical",
    "Properties": {
        "gravity":   768,
        "frictionX": 800,
        "frictionY": 0
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that destroys its own entity after a delay.
//
// Property    Type       Description
// --------    ----       -----------
// delay       number     Milliseconds to delay before destructing.
//
{"Component": "SelfDestructing", "Specializes": "TimerScript",
    "Properties": {
        "OnExpired": function() engine::Destroy(this); end function
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// The AI for a shot. It sends "Shot" messages to every entity it collides
// with, and then simply destructs when it collides with a wall or a
// ShotBlocker of a matching type.
//
// Property    Type       Description
// --------    ----       -----------
// shooter     string     The entity type of the shooter (e.g. Player, Enemy);
//                        used by the ShotBlocker component.
//
{"Component": "ShotAI", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "Collided":
                function(args : association)
                    engine::Send(args["eid"], "Shot", {"eid": this, "shooter": shooter});
                end function,
            "HitWall":
                function()
                    engine::Destroy(this);
                end function,
            "ShotBlocked":
                function()
                    engine::Destroy(this);
                end function
        }
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that indicates to a Shot that it has been blocked by the
// component's entity.
//
// Property    Type       Description
// --------    ----       -----------
// shooter     string     Blocks shots produced by this shooter type.
//
{"Component": "ShotBlocker", "Specializes": "MessageScript",
    "Properties": {
        "handlers": {
            "Shot":
                function(args : association)
                    if args["shooter"] = shooter then
                        engine::Send(args["eid"], "ShotBlocked");
                    end if;
                end function
        }
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that blocks shots from the Player entity.
//
{"Component": "PlayerShotBlocker", "Specializes": "ShotBlocker",
    "Properties": {
        "shooter": "Player"
    }
},

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that spawns a new entity, on entity destruction, at its
// current location (with an optional location offset).
//
// Property     Type          Description
// --------     ----          -----------
// template     string        The entity template to use
// offsetX      number        X offset of new entity (default=0)
// offsetY      number        Y offset of new entity (default=0)
// properties   association   Properties for entity template (default={})
//
{"Component": "SpawnOnDestruct", "Specializes": "OnDestruct",
    "Properties": {
        "offsetX": 0,
        "offsetY": 0,
        "properties": {},
        "OnDestruct":
            function()
                engine::SpawnAt(template, this.x + offsetX, this.y + offsetY, properties);
            end function
    }
}

]
