////////////////////////////////////////////////////////////////////////////////
//                              ORACLE ENGINE API                             //

////////////////////
// GAME FUNCTIONS //
////////////////////

// PlayMusic(name : string) returns null
//
// Stops the music, if any is playing, and begins playing music 'filename'. If
// a file extension is not specified, it will be detected using the supported
// audio file formats.
engine::PlayMusic := function(filename : string) : PlayMusic;

// StopMusic() returns null
//
// Stops the music, if any is playing.
engine::StopMusic := function() : StopMusic;

// PlaySound(filename : string) returns null
//
// Begins playing 'filename' once, as a sound effect. If a file extension is not
// specified, it will be detected using the supported audio formats.
engine::PlaySound := function(filename : string) : PlaySound;

/////////////////////
// WORLD FUNCTIONS //
/////////////////////

engine::MAP_BACKGROUND   := 1;      // background map layer
engine::MAP_MIDDLEGROUND := 2;      // middle map layer (still behind entities)
engine::MAP_FOREGROUND   := 3;      // foreground map layer (in front of entities)

// LoadWorld(filename : string) returns null
//
// Loads world 'filename' and begins playing. If a file extension is not
// specified, the standard world file extension will be used. This is a queued
// event; it will not occur immediately.
engine::LoadWorld := function(filename : string) : LoadWorld;

// GetTile(layer : number, x : number, y : number) returns number
//
// Returns the id of the tile in map layer 'layer', located at 'x', 'y'. The
// coordinates are given in world units (e.g. pixels). The MAP_xxx constants
// can be used to specify the layer.
engine::GetTile := function(layer : number, x : number, y : number) : GetTile;

// SetTile(layer : number, x : number, y : number, tileId : number) returns null
//
// Sets the id of the tile in map layer 'layer', located at 'x', 'y'. The
// coordinates are given in world units (e.g. pixels). The MAP_xxx constants
// can be used to specify the layer. Use tile id 0 to clear the map location.
engine::SetTile := function(layer : number, x : number, y : number, tileId : number) : SetTile;

//////////////////////
// ENTITY FUNCTIONS //
//////////////////////

// Spawn(template : string, properties : association) returns null
//
// Spawns a new entity, constructed with entity template 'template'.
// 'properties' contains entity instance-specific properties that are used to
// parameterize the entity template, such as entity location. This is a queued
// event; it will not occur immediately.
engine::Spawn := function(template : string, properties : association) : Spawn;

// SpawnAt(template : string, properties : association) returns null
//
// Spawns a new entity, constructed with entity template 'template'. This is a
// convenience function for specifying the new entity's location at 'x', 'y'.
// It is equivalent to calling:
//   Spawn(template, rmerge({"Location": {"x": x, "y": y}}, properties));
engine::SpawnAt := function(template : string, x : number, y : number, properties : association := {}) : SpawnAt;

// Destroy(target : id) returns null
//
// Destroys entity id 'target', if it exists. This is a queued event; it will
// not occur immediately.
engine::Destroy := function(target : id) : Destroy;

// EntityExists(entity : id) return boolean
//
// Returns True if 'entity' references an existing entity.
engine::EntityExists := function(entity : id) : EntityExists;

// RemoveComponent(entity : id, family : string) returns null
//
// Removes a component of family 'family' from 'entity', if it has one. It the
// entity has none, or more than one, nothing will change. Note that 'family' is
// the family of the component, not a component specialization name. See the
// list of built-in components for component family names.
engine::RemoveComponent := function(entity : id, family : string) : RemoveComponent;

// Send(target : id, message : string, params : association) returns null
//
// Sends a message 'message' to entity id 'target', if it exists. The message is
// parameterized with the contents of 'params'. The entity may or may not react
// to the message, depending on its own composition and implementation.
engine::Send := function(target : id, message : string, params : association := {}) : Send;

////////////////////////////////////////////////////////////////////////////////
//                               KEEN GALAXY API                              //

// EnterLevel(name : string)
//
// Enters the new level 'name' from the map world. game::lastWin will be cleared.
engine::EnterLevel :=
    function(name : string)
        engine::StopMusic();
        engine::PlaySound("enter_level");
        engine::LoadWorld(name);
    end function;

// ExitLevel()
//
// Exits the current level for the map world, marking it as completed.
// game::lastWin will be set, and game::completed will be updated.
engine::ExitLevel :=
    function()
        engine::StopMusic();
        engine::PlaySound("exit_level");
        engine::LoadWorld("");
    end function;

// MessageBox(text : string, icon : string, align : string)
//
// Pauses the game and displays a message box on the screen. If an icon is
// displayed (e.g. a talking character) then 'align' specifies the side of the
// box where the text is displayed.
engine::MessageBox :=
    function(text  : string,
             icon  : string := "",
             align : string := "right") : MessageBox;

